      subroutine ebirate3(ny,nr,r,gain,loss)
c
c----CAMx v4.53 090209
c
c     EBIRATE3 computes species production and loss
c     for the EBI solver
c
c     Copyright 1996-2009
c     ENVIRON International Corporation
c     Created by the CMC version 5
c
c --- Subroutines Called:
c        none
c
c --- Called by:
c        EBISOLV
c
c --- Argument definitions:
c        ny   - dimension of gain and loss
c        nr   - dimension of r
c        r    - reaction rates (hr-1)
c        gain - species production (ppm/hr)
c        loss - species destruction (ppm/hr)
c
c --- Includes:
      implicit none
      include "camx.prm"
      include "ddmchm.com"
c
c --- Arguments:
      integer ny, nr, l
      real    loss(ny+1), gain(ny+1)
      real    r(nr)
c
c --- Entry Point:
c
c
c-----Calculate the species rates
c
c
c   NO2    NO     O    O3   NO3   O1D    OH   HO2  N2O5  HNO3
c  HONO   PNA  H2O2    CO  FORM  ALD2  C2O3   XO2   PAN   PAR
c  XO2N   ROR   NTR   OLE   ETH   TOL  CRES   TO2  OPEN   CRO
c  MGLY   XYL  ISOP  ISPD   SO2  SULF  MEOH  ETOH
c
        Loss(lNO2  )= +( 1.000)*r(  1)+( 1.000)*r(  4)+( 1.000)*r(  5)
     &                +( 1.000)*r(  7)+( 1.000)*r( 16)+( 1.000)*r( 17)
     &                +( 1.000)*r( 21)+( 1.000)*r( 26)+( 1.000)*r( 29)
     &                +( 1.000)*r( 47)+( 1.000)*r( 55)+( 1.000)*r( 68)
     &                +( 1.000)*r( 96)
c
        Gain(lNO2  )= +( 1.000)*r(  3)+( 1.000)*r(  6)+( 0.890)*r( 14)
     &                +( 2.000)*r( 15)+( 1.000)*r( 16)+( 1.000)*r( 19)
     &                +( 2.000)*r( 20)+( 1.000)*r( 24)+( 1.000)*r( 25)
     &                +( 1.000)*r( 28)+( 1.000)*r( 30)+( 1.000)*r( 31)
     &                +( 1.000)*r( 46)+( 1.000)*r( 48)+( 1.000)*r( 59)
     &                +( 0.900)*r( 64)+( 0.200)*r( 78)+( 1.000)*r( 79)
c
        Loss(lNO   )= +( 1.000)*r(  3)+( 1.000)*r(  6)+( 1.000)*r( 15)
     &                +( 2.000)*r( 20)+( 1.000)*r( 21)+( 1.000)*r( 22)
     &                +( 1.000)*r( 28)+( 1.000)*r( 46)+( 1.000)*r( 64)
     &                +( 1.000)*r( 79)+( 1.000)*r( 81)
c
        Gain(lNO   )= +( 1.000)*r(  1)+( 1.000)*r(  4)+( 0.110)*r( 14)
     &                +( 1.000)*r( 16)+( 1.000)*r( 23)+( 1.000)*r( 25)
     &                +( 0.200)*r( 96)
c
        Loss(lO    )= +( 1.000)*r(  2)+( 1.000)*r(  4)+( 1.000)*r(  5)
     &                +( 1.000)*r(  6)+( 1.000)*r( 40)+( 1.000)*r( 42)
     &                +( 1.000)*r( 56)+( 1.000)*r( 60)+( 1.000)*r( 75)
c
        Gain(lO    )= +( 1.000)*r(  1)+( 1.000)*r(  8)+( 1.000)*r( 10)
     &                +( 0.890)*r( 14)
c
        Loss(lO3   )= +( 1.000)*r(  3)+( 1.000)*r(  7)+( 1.000)*r(  8)
     &                +( 1.000)*r(  9)+( 1.000)*r( 12)+( 1.000)*r( 13)
     &                +( 1.000)*r( 58)+( 1.000)*r( 62)+( 1.000)*r( 71)
     &                +( 1.000)*r( 77)+( 1.000)*r( 93)
c
        Gain(lO3   )= +( 1.000)*r(  2)
c
        Loss(lNO3  )= +( 1.000)*r( 14)+( 1.000)*r( 15)+( 1.000)*r( 16)
     &                +( 1.000)*r( 17)+( 1.000)*r( 41)+( 1.000)*r( 44)
     &                +( 1.000)*r( 59)+( 1.000)*r( 67)+( 1.000)*r( 78)
     &                +( 1.000)*r( 94)
c
        Gain(lNO3  )= +( 1.000)*r(  5)+( 1.000)*r(  7)+( 1.000)*r( 19)
     &                +( 1.000)*r( 27)
c
        Loss(lO1D  )= +( 1.000)*r( 10)+( 1.000)*r( 11)
c
        Gain(lO1D  )= +( 1.000)*r(  9)
c
        Loss(lOH   )= +( 1.000)*r( 12)+( 1.000)*r( 22)+( 1.000)*r( 24)
     &                +( 1.000)*r( 26)+( 1.000)*r( 27)+( 1.000)*r( 31)
     &                +( 1.000)*r( 35)+( 1.000)*r( 36)+( 1.000)*r( 37)
     &                +( 1.000)*r( 43)+( 1.000)*r( 51)+( 1.000)*r( 52)
     &                +( 1.000)*r( 57)+( 1.000)*r( 61)+( 1.000)*r( 63)
     &                +( 1.000)*r( 66)+( 1.000)*r( 70)+( 1.000)*r( 72)
     &                +( 1.000)*r( 73)+( 1.000)*r( 76)+( 1.000)*r( 82)
     &                +( 1.000)*r( 84)+( 1.000)*r( 85)+( 1.000)*r( 90)
     &                +( 1.000)*r( 92)
c
        Gain(lOH   )= +( 2.000)*r( 11)+( 1.000)*r( 13)+( 1.000)*r( 23)
     &                +( 1.000)*r( 28)+( 2.000)*r( 34)+( 1.000)*r( 40)
     &                +( 1.000)*r( 42)+( 0.790)*r( 50)+( 0.200)*r( 56)
     &                +( 0.100)*r( 58)+( 0.300)*r( 60)+( 0.080)*r( 71)
     &                +( 0.266)*r( 77)+( 0.268)*r( 93)
c
        Loss(lHO2  )= +( 1.000)*r( 13)+( 1.000)*r( 28)+( 1.000)*r( 29)
     &                +( 2.000)*r( 32)+( 2.000)*r( 33)+( 1.000)*r( 50)
     &                +( 1.000)*r( 86)+( 1.000)*r( 87)+( 1.000)*r( 90)
c
        Gain(lHO2  )= +( 1.000)*r( 12)+( 1.000)*r( 30)+( 1.000)*r( 35)
     &                +( 1.000)*r( 36)+( 1.000)*r( 37)+( 2.000)*r( 38)
     &                +( 1.000)*r( 40)+( 1.000)*r( 41)+( 2.000)*r( 45)
     &                +( 1.000)*r( 46)+( 2.000)*r( 49)+( 0.790)*r( 50)
     &                +( 1.000)*r( 51)+( 0.110)*r( 52)+( 0.940)*r( 53)
     &                +( 1.000)*r( 54)+( 0.380)*r( 56)+( 1.000)*r( 57)
     &                +( 0.440)*r( 58)+( 1.700)*r( 60)+( 1.000)*r( 61)
     &                +( 0.120)*r( 62)+( 0.440)*r( 63)+( 0.900)*r( 64)
     &                +( 1.000)*r( 65)+( 0.600)*r( 66)+( 1.000)*r( 69)
        Gain(lHO2  ) = Gain(lHO2  )
     &                +( 2.000)*r( 70)+( 0.760)*r( 71)+( 0.700)*r( 72)
     &                +( 1.000)*r( 74)+( 0.250)*r( 75)+( 0.912)*r( 76)
     &                +( 0.066)*r( 77)+( 0.800)*r( 78)+( 1.000)*r( 82)
     &                +( 1.000)*r( 84)+( 1.000)*r( 85)+( 0.503)*r( 92)
     &                +( 0.154)*r( 93)+( 0.925)*r( 94)+( 1.033)*r( 95)
     &                +( 0.800)*r( 96)
c
        Loss(lN2O5 )= +( 1.000)*r( 18)+( 1.000)*r( 19)
c
        Gain(lN2O5 )= +( 1.000)*r( 17)
c
        Loss(lHNO3 )= +( 1.000)*r( 27)
c
        Gain(lHNO3 )= +( 2.000)*r( 18)+( 1.000)*r( 26)+( 1.000)*r( 41)
     &                +( 1.000)*r( 44)+( 1.000)*r( 67)+( 0.150)*r( 94)
c
        Loss(lHONO )= +( 1.000)*r( 23)+( 1.000)*r( 24)+( 2.000)*r( 25)
c
        Gain(lHONO )= +( 2.000)*r( 21)+( 1.000)*r( 22)
c
        Loss(lPNA  )= +( 1.000)*r( 30)+( 1.000)*r( 31)
c
        Gain(lPNA  )= +( 1.000)*r( 29)
c
        Loss(lH2O2 )= +( 1.000)*r( 34)+( 1.000)*r( 35)
c
        Gain(lH2O2 )= +( 1.000)*r( 32)+( 1.000)*r( 33)
c
        Loss(lCO   )= +( 1.000)*r( 36)
c
        Gain(lCO   )= +( 1.000)*r( 37)+( 1.000)*r( 38)+( 1.000)*r( 39)
     &                +( 1.000)*r( 40)+( 1.000)*r( 41)+( 1.000)*r( 45)
     &                +( 0.300)*r( 56)+( 0.330)*r( 58)+( 1.000)*r( 60)
     &                +( 0.420)*r( 62)+( 1.000)*r( 69)+( 2.000)*r( 70)
     &                +( 0.690)*r( 71)+( 1.000)*r( 74)+( 0.066)*r( 77)
     &                +( 0.334)*r( 92)+( 0.225)*r( 93)+( 0.643)*r( 94)
     &                +( 0.333)*r( 95)
c
        Loss(lFORM )= +( 1.000)*r( 37)+( 1.000)*r( 38)+( 1.000)*r( 39)
     &                +( 1.000)*r( 40)+( 1.000)*r( 41)
c
        Gain(lFORM )= +( 1.000)*r( 45)+( 1.000)*r( 46)+( 2.000)*r( 49)
     &                +( 0.790)*r( 50)+( 1.000)*r( 51)+( 0.200)*r( 56)
     &                +( 1.000)*r( 57)+( 0.740)*r( 58)+( 1.000)*r( 59)
     &                +( 1.000)*r( 60)+( 1.560)*r( 61)+( 1.000)*r( 62)
     &                +( 1.000)*r( 70)+( 0.700)*r( 71)+( 0.500)*r( 75)
     &                +( 0.629)*r( 76)+( 0.600)*r( 77)+( 1.000)*r( 84)
     &                +( 0.167)*r( 92)+( 0.150)*r( 93)+( 0.282)*r( 94)
     &                +( 0.900)*r( 95)
c
        Loss(lALD2 )= +( 1.000)*r( 42)+( 1.000)*r( 43)+( 1.000)*r( 44)
     &                +( 1.000)*r( 45)
c
        Gain(lALD2 )= +( 0.110)*r( 52)+( 1.100)*r( 53)+( 0.630)*r( 56)
     &                +( 1.000)*r( 57)+( 0.500)*r( 58)+( 1.000)*r( 59)
     &                +( 0.220)*r( 61)+( 0.030)*r( 71)+( 0.150)*r( 77)
     &                +( 0.800)*r( 78)+( 1.000)*r( 85)+( 0.273)*r( 92)
     &                +( 0.020)*r( 93)+( 0.357)*r( 94)+( 0.067)*r( 95)
     &                +( 0.800)*r( 96)
c
        Loss(lC2O3 )= +( 1.000)*r( 46)+( 1.000)*r( 47)+( 2.000)*r( 49)
     &                +( 1.000)*r( 50)
c
        Gain(lC2O3 )= +( 1.000)*r( 42)+( 1.000)*r( 43)+( 1.000)*r( 44)
     &                +( 1.000)*r( 48)+( 1.000)*r( 69)+( 1.000)*r( 70)
     &                +( 0.620)*r( 71)+( 1.000)*r( 73)+( 1.000)*r( 74)
     &                +( 0.250)*r( 75)+( 0.200)*r( 77)+( 0.498)*r( 92)
     &                +( 0.114)*r( 93)+( 0.075)*r( 94)+( 0.967)*r( 95)
c
        Loss(lXO2  )= +( 1.000)*r( 79)+( 2.000)*r( 80)+( 1.000)*r( 86)
     &                +( 1.000)*r( 89)
c
        Gain(lXO2  )= +( 1.000)*r( 45)+( 1.000)*r( 46)+( 2.000)*r( 49)
     &                +( 0.790)*r( 50)+( 1.000)*r( 51)+( 0.870)*r( 52)
     &                +( 0.960)*r( 53)+( 0.280)*r( 56)+( 1.000)*r( 57)
     &                +( 0.220)*r( 58)+( 0.910)*r( 59)+( 0.700)*r( 60)
     &                +( 1.000)*r( 61)+( 0.080)*r( 63)+( 0.600)*r( 66)
     &                +( 1.000)*r( 70)+( 0.030)*r( 71)+( 0.500)*r( 72)
     &                +( 1.000)*r( 73)+( 0.250)*r( 75)+( 0.991)*r( 76)
     &                +( 0.200)*r( 77)+( 1.000)*r( 78)+( 0.713)*r( 92)
     &                +( 0.064)*r( 93)+( 0.075)*r( 94)+( 0.700)*r( 95)
        Gain(lXO2  ) = Gain(lXO2  )
     &                +( 1.000)*r( 96)
c
        Loss(lPAN  )= +( 1.000)*r( 48)
c
        Gain(lPAN  )= +( 1.000)*r( 47)
c
        Loss(lPAR  )= +( 1.000)*r( 52)
c
        Gain(lPAR  )= +(-0.110)*r( 52)+(-2.100)*r( 53)+( 0.220)*r( 56)
     &                +(-1.000)*r( 57)+(-1.000)*r( 58)+(-1.000)*r( 59)
     &                +( 1.100)*r( 72)+( 0.250)*r( 75)+( 0.350)*r( 77)
     &                +( 2.400)*r( 78)+( 1.565)*r( 92)+( 0.360)*r( 93)
     &                +( 1.282)*r( 94)+( 0.832)*r( 95)+( 2.400)*r( 96)
c
        Loss(lXO2N )= +( 1.000)*r( 81)+( 1.000)*r( 87)+( 2.000)*r( 88)
     &                +( 1.000)*r( 89)
c
        Gain(lXO2N )= +( 0.130)*r( 52)+( 0.040)*r( 53)+( 0.020)*r( 56)
     &                +( 0.090)*r( 59)+( 0.088)*r( 76)
c
        Loss(lROR  )= +( 1.000)*r( 53)+( 1.000)*r( 54)+( 1.000)*r( 55)
c
        Gain(lROR  )= +( 0.760)*r( 52)
c
        Loss(lNTR  )= 0.0
c
        Gain(lNTR  )= +( 1.000)*r( 55)+( 0.100)*r( 64)+( 1.000)*r( 68)
     &                +( 0.800)*r( 78)+( 1.000)*r( 81)+( 0.850)*r( 94)
     &                +( 0.800)*r( 96)
c
        Loss(lOLE  )= +( 1.000)*r( 56)+( 1.000)*r( 57)+( 1.000)*r( 58)
     &                +( 1.000)*r( 59)
c
        Gain(lOLE  )= 0.0
c
        Loss(lETH  )= +( 1.000)*r( 60)+( 1.000)*r( 61)+( 1.000)*r( 62)
c
        Gain(lETH  )= 0.0
c
        Loss(lTOL  )= +( 1.000)*r( 63)
c
        Gain(lTOL  )= 0.0
c
        Loss(lCRES )= +( 1.000)*r( 66)+( 1.000)*r( 67)
c
        Gain(lCRES )= +( 0.360)*r( 63)+( 1.000)*r( 65)+( 0.200)*r( 72)
c
        Loss(lTO2  )= +( 1.000)*r( 64)+( 1.000)*r( 65)
c
        Gain(lTO2  )= +( 0.560)*r( 63)+( 0.300)*r( 72)
c
        Loss(lOPEN )= +( 1.000)*r( 69)+( 1.000)*r( 70)+( 1.000)*r( 71)
c
        Gain(lOPEN )= +( 0.900)*r( 64)+( 0.300)*r( 66)
c
        Loss(lCRO  )= +( 1.000)*r( 68)+( 1.000)*r( 91)
c
        Gain(lCRO  )= +( 0.400)*r( 66)+( 1.000)*r( 67)
c
        Loss(lMGLY )= +( 1.000)*r( 73)+( 1.000)*r( 74)
c
        Gain(lMGLY )= +( 0.200)*r( 71)+( 0.800)*r( 72)+( 0.168)*r( 92)
     &                +( 0.850)*r( 93)
c
        Loss(lXYL  )= +( 1.000)*r( 72)
c
        Gain(lXYL  )= 0.0
c
        Loss(lISOP )= +( 1.000)*r( 75)+( 1.000)*r( 76)+( 1.000)*r( 77)
     &                +( 1.000)*r( 78)+( 1.000)*r( 96)
c
        Gain(lISOP )= 0.0
c
        Loss(lISPD )= +( 1.000)*r( 92)+( 1.000)*r( 93)+( 1.000)*r( 94)
     &                +( 1.000)*r( 95)
c
        Gain(lISPD )= +( 0.750)*r( 75)+( 0.912)*r( 76)+( 0.650)*r( 77)
     &                +( 0.200)*r( 78)+( 0.200)*r( 96)
c
        Loss(lSO2  )= +( 1.000)*r( 82)+( 1.000)*r( 83)
c
        Gain(lSO2  )= 0.0
c
        Loss(lSULF )= 0.0
c
        Gain(lSULF )= +( 1.000)*r( 82)+( 1.000)*r( 83)
c
        Loss(lMEOH )= +( 1.000)*r( 84)
c
        Gain(lMEOH )= 0.0
c
        Loss(lETOH )= +( 1.000)*r( 85)
c
        Gain(lETOH )= 0.0
c
c
      return
      end

      subroutine ebirxn3(ny,nr,yh,H2O,M,O2,CH4,H2,rk,r)
c
c----CAMx v4.53 090209
c
c     EBIRXN3 computes reaction rates for the EBI solver
c
c     Copyright 1996-2009
c     ENVIRON International Corporation
c     Created by the CMC version 5
c
c --- Subroutines Called:
c        none
c
c --- Called by:
c        EBISOLV
c
c --- Argument definitions:
c        ny  - dimension of y
c        nr  - dimension of r and rk
c        yh  - current y (ppm)
c        H2O - water vapor Concentration (ppm)
c        M   - total gas Concentration (ppm)
c        O2  - oxygen Concentration (ppm)
c        CH4 - methane Concentration (ppm)
c        H2  - hydrogen Concentration (ppm)
c        rk  - rate constants (units ppm hr)
c        r   - reaction rates (hr-1)
c
c --- Includes:
      implicit none
      include "camx.prm"
      include "ddmchm.com"
c
c --- Arguments:
      integer ny, nr
      real    yh(ny+1)
      real    r(nr), rk(nr)
      real    H2O,M,O2,CH4,H2
c
c --- Local variables:
      real    N2
c
c --- Entry point
c
      N2  = M - O2
c
c --- Calculate reaction rates
c
      r(  1) = rk(  1)*yh(lNO2)
      r(  2) = rk(  2)*yh(lO)
      r(  3) = rk(  3)*yh(lO3)*yh(lNO)
      r(  4) = rk(  4)*yh(lO)*yh(lNO2)
      r(  5) = rk(  5)*yh(lO)*yh(lNO2)
      r(  6) = rk(  6)*yh(lO)*yh(lNO)
      r(  7) = rk(  7)*yh(lNO2)*yh(lO3)
      r(  8) = rk(  8)*yh(lO3)
      r(  9) = rk(  9)*yh(lO3)
      r( 10) = rk( 10)*yh(lO1D)
      r( 11) = rk( 11)*yh(lO1D)*H2O
      r( 12) = rk( 12)*yh(lO3)*yh(lOH)
      r( 13) = rk( 13)*yh(lO3)*yh(lHO2)
      r( 14) = rk( 14)*yh(lNO3)
      r( 15) = rk( 15)*yh(lNO3)*yh(lNO)
      r( 16) = rk( 16)*yh(lNO3)*yh(lNO2)
      r( 17) = rk( 17)*yh(lNO3)*yh(lNO2)
      r( 18) = rk( 18)*yh(lN2O5)*H2O
      r( 19) = rk( 19)*yh(lN2O5)
      r( 20) = rk( 20)*yh(lNO)*yh(lNO)
      r( 21) = rk( 21)*yh(lNO)*yh(lNO2)*H2O
      r( 22) = rk( 22)*yh(lNO)*yh(lOH)
      r( 23) = rk( 23)*yh(lHONO)
      r( 24) = rk( 24)*yh(lOH)*yh(lHONO)
      r( 25) = rk( 25)*yh(lHONO)*yh(lHONO)
      r( 26) = rk( 26)*yh(lNO2)*yh(lOH)
      r( 27) = rk( 27)*yh(lOH)*yh(lHNO3)
      r( 28) = rk( 28)*yh(lHO2)*yh(lNO)
      r( 29) = rk( 29)*yh(lHO2)*yh(lNO2)
      r( 30) = rk( 30)*yh(lPNA)
      r( 31) = rk( 31)*yh(lOH)*yh(lPNA)
      r( 32) = rk( 32)*yh(lHO2)*yh(lHO2)
      r( 33) = rk( 33)*yh(lHO2)*yh(lHO2)*H2O
      r( 34) = rk( 34)*yh(lH2O2)
      r( 35) = rk( 35)*yh(lOH)*yh(lH2O2)
      r( 36) = rk( 36)*yh(lOH)*yh(lCO)
      r( 37) = rk( 37)*yh(lFORM)*yh(lOH)
      r( 38) = rk( 38)*yh(lFORM)
      r( 39) = rk( 39)*yh(lFORM)
      r( 40) = rk( 40)*yh(lFORM)*yh(lO)
      r( 41) = rk( 41)*yh(lFORM)*yh(lNO3)
      r( 42) = rk( 42)*yh(lALD2)*yh(lO)
      r( 43) = rk( 43)*yh(lALD2)*yh(lOH)
      r( 44) = rk( 44)*yh(lALD2)*yh(lNO3)
      r( 45) = rk( 45)*yh(lALD2)
      r( 46) = rk( 46)*yh(lC2O3)*yh(lNO)
      r( 47) = rk( 47)*yh(lC2O3)*yh(lNO2)
      r( 48) = rk( 48)*yh(lPAN)
      r( 49) = rk( 49)*yh(lC2O3)*yh(lC2O3)
      r( 50) = rk( 50)*yh(lC2O3)*yh(lHO2)
      r( 51) = rk( 51)*yh(lOH)
      r( 52) = rk( 52)*yh(lPAR)*yh(lOH)
      r( 53) = rk( 53)*yh(lROR)
      r( 54) = rk( 54)*yh(lROR)
      r( 55) = rk( 55)*yh(lROR)*yh(lNO2)
      r( 56) = rk( 56)*yh(lO)*yh(lOLE)
      r( 57) = rk( 57)*yh(lOH)*yh(lOLE)
      r( 58) = rk( 58)*yh(lO3)*yh(lOLE)
      r( 59) = rk( 59)*yh(lNO3)*yh(lOLE)
      r( 60) = rk( 60)*yh(lO)*yh(lETH)
      r( 61) = rk( 61)*yh(lOH)*yh(lETH)
      r( 62) = rk( 62)*yh(lO3)*yh(lETH)
      r( 63) = rk( 63)*yh(lTOL)*yh(lOH)
      r( 64) = rk( 64)*yh(lTO2)*yh(lNO)
      r( 65) = rk( 65)*yh(lTO2)
      r( 66) = rk( 66)*yh(lOH)*yh(lCRES)
      r( 67) = rk( 67)*yh(lCRES)*yh(lNO3)
      r( 68) = rk( 68)*yh(lCRO)*yh(lNO2)
      r( 69) = rk( 69)*yh(lOPEN)
      r( 70) = rk( 70)*yh(lOPEN)*yh(lOH)
      r( 71) = rk( 71)*yh(lOPEN)*yh(lO3)
      r( 72) = rk( 72)*yh(lOH)*yh(lXYL)
      r( 73) = rk( 73)*yh(lOH)*yh(lMGLY)
      r( 74) = rk( 74)*yh(lMGLY)
      r( 75) = rk( 75)*yh(lO)*yh(lISOP)
      r( 76) = rk( 76)*yh(lOH)*yh(lISOP)
      r( 77) = rk( 77)*yh(lO3)*yh(lISOP)
      r( 78) = rk( 78)*yh(lNO3)*yh(lISOP)
      r( 79) = rk( 79)*yh(lXO2)*yh(lNO)
      r( 80) = rk( 80)*yh(lXO2)*yh(lXO2)
      r( 81) = rk( 81)*yh(lXO2N)*yh(lNO)
      r( 82) = rk( 82)*yh(lSO2)*yh(lOH)
      r( 83) = rk( 83)*yh(lSO2)
      r( 84) = rk( 84)*yh(lMEOH)*yh(lOH)
      r( 85) = rk( 85)*yh(lETOH)*yh(lOH)
      r( 86) = rk( 86)*yh(lXO2)*yh(lHO2)
      r( 87) = rk( 87)*yh(lXO2N)*yh(lHO2)
      r( 88) = rk( 88)*yh(lXO2N)*yh(lXO2N)
      r( 89) = rk( 89)*yh(lXO2)*yh(lXO2N)
      r( 90) = rk( 90)*yh(lOH)*yh(lHO2)
      r( 91) = rk( 91)*yh(lCRO)
      r( 92) = rk( 92)*yh(lOH)*yh(lISPD)
      r( 93) = rk( 93)*yh(lO3)*yh(lISPD)
      r( 94) = rk( 94)*yh(lNO3)*yh(lISPD)
      r( 95) = rk( 95)*yh(lISPD)
      r( 96) = rk( 96)*yh(lNO2)*yh(lISOP)
c
      return
      end

      subroutine hr_hox3(y0,y1,yh,H2O,M,O2,CH4,H2,ny,rk,r,nr,dt)
c
c----CAMx v4.53 090209
c
c     HR_HOX3 solves the HOx family using Hertel's equations
c
c     Copyright 1996-2009
c     ENVIRON International Corporation
c     Created by the CMC version 5
c
c --- Subroutines Called:
c        none
c
c --- Called by:
c        EBISOLV
c
c --- Argument definitions:
c        y0  - initial Conc for this step (ppm)
c        yh  - current Conc iteration (ppm)
c        y1  - next Conc iteration (ppm)
c        H2O - water vapor Conc (ppm)
c        M   - total gas Conc (ppm)
c        O2  - oxygen Conc (ppm)
c        CH4 - methane Conc (ppm)
c        H2  - hydrogen Conc (ppm)
c        ny  - dimension of y0, y1 and yh
c        rk  - rate constants (ppm-n hr-1)
c        r   - reaction rates (hr-1)
c        nr  - dimension of rk and r
c        dt  - time step (hr)
c
c --- Includes:
      implicit none
      include "camx.prm"
      include "ddmchm.com"
c
c --- Arguments:
      integer ny, nr
      real y0(ny+1), y1(ny+1), yh(ny+1)
      real rk(nr), r(nr)
      real H2O, M, O2, CH4, H2, dt
c
c --- Local variables:
      real N2
      real t1, t2, t3, A, B, C, Q
      real newOH, newHO2, newHONO, newPNA
      real lsOH, lsHO2, lsHONO, lsPNA, lsO1D, yldOH, self
      real rOH_HO2, rHO2_OH, rOH_HONO, rHONO_OH, rHO2_PNA, rPNA_HO2
c
c --- Entry Point
c
      N2 = M - O2
c
c --- O1D loss
c
      lsO1D = 
     &       + rk( 10)
     &       + rk( 11)*H2O
c
c --- Yield of OH from O1D
c
      yldOH  = ( 
     &         ( 2.000)*rk( 11)*H2O ) / lsO1D
c
c --- Conversion of HONO to OH
c
      rHONO_OH  = dt*(
     &       +          rk( 23) )
c
c --- Conversion of HO2 to OH
c
      rHO2_OH  = dt*(
     &       +          rk( 13)*yh(lO3)
     &       +          rk( 28)*yh(lNO)
     &       + ( 0.790)*rk( 50)*yh(lC2O3) )
c
c --- Other OH production terms
c
      newOH = y0(lOH) + dt*(
     &       + ( 2.000)*r( 34)
     &       +          r( 40)
     &       +          r( 42)
     &       + ( 0.200)*r( 56)
     &       + ( 0.100)*r( 58)
     &       + ( 0.300)*r( 60)
     &       + ( 0.080)*r( 71)
     &       + ( 0.266)*r( 77)
     &       + ( 0.268)*r( 93)
     &       +          r(  9)*yldOH )
c
c --- Conversion of PNA to HO2
c
      rPNA_HO2  = dt*(
     &       +          rk( 30) )
c
c --- Conversion of OH to HO2
c
      rOH_HO2  = dt*(
     &       +          rk( 12)*yh(lO3)
     &       +          rk( 35)*yh(lH2O2)
     &       +          rk( 36)*yh(lCO)
     &       +          rk( 37)*yh(lFORM)
     &       +          rk( 51)
     &       + ( 0.110)*rk( 52)*yh(lPAR)
     &       +          rk( 57)*yh(lOLE)
     &       +          rk( 61)*yh(lETH)
     &       + ( 0.440)*rk( 63)*yh(lTOL)
     &       + ( 0.600)*rk( 66)*yh(lCRES)
     &       + ( 2.000)*rk( 70)*yh(lOPEN)
     &       + ( 0.700)*rk( 72)*yh(lXYL)
     &       + ( 0.912)*rk( 76)*yh(lISOP)
     &       +          rk( 82)*yh(lSO2)
     &       +          rk( 84)*yh(lMEOH)
     &       +          rk( 85)*yh(lETOH)
     &       + ( 0.503)*rk( 92)*yh(lISPD) )
c
c --- Other HO2 production terms
c
      newHO2 = y0(lHO2) + dt*(
     &       + ( 2.000)*r( 38)
     &       +          r( 40)
     &       +          r( 41)
     &       + ( 2.000)*r( 45)
     &       +          r( 46)
     &       + ( 2.000)*r( 49)
     &       + ( 0.940)*r( 53)
     &       +          r( 54)
     &       + ( 0.380)*r( 56)
     &       + ( 0.440)*r( 58)
     &       + ( 1.700)*r( 60)
     &       + ( 0.120)*r( 62)
     &       + ( 0.900)*r( 64)
     &       +          r( 65)
     &       +          r( 69)
     &       + ( 0.760)*r( 71)
     &       +          r( 74)
     &       + ( 0.250)*r( 75)
     &       + ( 0.066)*r( 77)
     &       + ( 0.800)*r( 78)
     &       + ( 0.154)*r( 93)
     &       + ( 0.925)*r( 94)
     &       + ( 1.033)*r( 95)
     &       + ( 0.800)*r( 96) )
c
c
c --- Conversion of OH to HONO
c
      rOH_HONO  = dt*(
     &       +          rk( 22)*yh(lNO) )
c
c --- Other HONO production terms
c
      newHONO = y0(lHONO) + dt*(
     &       + ( 2.000)*r( 21) ) 
c
c --- Conversion of HO2 to PNA
c
      rHO2_PNA  = dt*(
     &       +          rk( 29)*yh(lNO2) )
c
c --- Other PNA production terms
c
      newPNA = y0(lPNA) 
c
c --- Net loss of OH
c
      lsOH = 1.0 + dt*(
     &       +          rk( 12)*yh(lO3)
     &       +          rk( 22)*yh(lNO)
     &       +          rk( 24)*yh(lHONO)
     &       +          rk( 26)*yh(lNO2)
     &       +          rk( 27)*yh(lHNO3)
     &       +          rk( 31)*yh(lPNA)
     &       +          rk( 35)*yh(lH2O2)
     &       +          rk( 36)*yh(lCO)
     &       +          rk( 37)*yh(lFORM)
     &       +          rk( 43)*yh(lALD2)
     &       +          rk( 51)
     &       +          rk( 52)*yh(lPAR)
     &       +          rk( 57)*yh(lOLE)
     &       +          rk( 61)*yh(lETH)
     &       +          rk( 63)*yh(lTOL)
     &       +          rk( 66)*yh(lCRES)
     &       +          rk( 70)*yh(lOPEN)
     &       +          rk( 72)*yh(lXYL)
     &       +          rk( 73)*yh(lMGLY)
     &       +          rk( 76)*yh(lISOP)
     &       +          rk( 82)*yh(lSO2)
     &       +          rk( 84)*yh(lMEOH)
     &       +          rk( 85)*yh(lETOH)
     &       +          rk( 90)*yh(lHO2)
     &       +          rk( 92)*yh(lISPD) )
c
c --- Loss of HO2, excluding self-reaction
c     (net either HO2 or OH produced)
c
      lsHO2 = 1.0 + rHO2_OH + rHO2_PNA + dt*(
     &       +          rk( 86)*yh(lXO2)
     &       +          rk( 87)*yh(lXO2N)
     &       +          rk( 90)*yh(lOH) )
c
c --- HO2 self-reaction
c
      self = dt*2.0*( 
     &       +          rk( 32)
     &       +          rk( 33)*H2O )
c
c --- Loss of HONO
c
      lsHONO = 1.0 + dt*(
     &       +          rk( 23)
     &       +          rk( 24)*yh(lOH)
     &       + ( 2.000)*rk( 25)*yh(lHONO) )
c
c --- Loss of PNA
c
      lsPNA = 1.0 + dt*(
     &       +          rk( 30)
     &       +          rk( 31)*yh(lOH) )
c
c --- Collect common terms
c
      t1 = 1.0 / ( lsOH*lsHONO - rHONO_OH*rOH_HONO )
      t2 = rOH_HO2*t1
      t3 = rPNA_HO2 / lsPNA
c
c --- Solve for HO2
c
      A = self
      B = lsHO2 - t3*rHO2_PNA - t2*rHO2_OH*lsHONO
      C = newHO2 + t3 * newPNA + t2*( newOH*lsHONO + newHONO*rHONO_OH )
      Q = -0.5 * (B + SIGN(1.0,B)*SQRT(B*B + 4.0*A*C))
c
c --- Update Concentrations
c
      y1(lHO2)  = MAX(1.0E-25, MAX(Q/A ,-C/Q) )
      y1(lOH)   = MAX(1.0E-25, ( ( newOH + rHO2_OH*y1(lHO2) )*lsHONO + 
     &                                        rHONO_OH*newHONO ) * t1 )
      y1(lPNA)  = MAX(1.0E-25, ( newPNA + rHO2_PNA*y1(lHO2) ) / lsPNA )
      y1(lHONO) = MAX(1.0E-25, ( newHONO + rOH_HONO*y1(lOH) ) / lsHONO )
c
      return
      end

      subroutine hr_nox3(y0,y1,yh,H2O,M,O2,CH4,H2,ny,rk,r,nr,dt)
c
c----CAMx v4.53 090209
c
c     HR_NOX3 solves the NOx family using Hertel's equations
c
c     Copyright 1996-2009
c     ENVIRON International Corporation
c     Created by the CMC version 5
c
c --- Subroutines Called:
c        none
c
c --- Called by:
c        EBISOLV
c
c --- Argument definitions:
c        y0  - initial Conc for this step (ppm)
c        yh  - current Conc iteration (ppm)
c        y1  - next Conc iteration (ppm)
c        H2O - water vapor Conc (ppm)
c        M   - total gas Conc (ppm)
c        O2  - oxygen Conc (ppm)
c        CH4 - methane Conc (ppm)
c        H2  - hydrogen Conc (ppm)
c        ny  - dimension of y0, y1 and yh
c        rk  - rate constants (ppm-n hr-1)
c        r   - reaction rates (hr-1)
c        nr  - dimension of rk and r
c        dt  - time step (hr)
c
c --- Includes:
      implicit none
      include "camx.prm"
      include "ddmchm.com"
c
c --- Arguments:
      integer ny, nr
      real y0(ny+1), y1(ny+1), yh(ny+1)
      real rk(nr), r(nr)
      real H2O, M, O2, CH4, H2, dt
c
c --- Local variables:
      real N2
      real jO3_O1D, rNO2_O3P, rO3P_O3, rO3wNO
      real totO1D, totNO, totNO2, totO3P, totO3
      real lsO1D, lsNO, lsNO2, lsO3P, lsO3
      real yldO3P, rNO_NO2, rNO2_NO
      real t1, t2, t3, t4, t5, f1, f2, f3, f4, s1, s2, A, B, C, Q
c
c --- Entry Point
c
      N2 = M - O2
c
c --- O1D loss
c
      lsO1D = 
     &       + rk( 10)
     &       + rk( 11)*H2O
c
c --- Yield of O3P from O1D
c
      yldO3P  = ( rk( 10) ) / lsO1D
c
c --- Conversion of NO2 to NO
c     (rNO2_NO = rate of NO2 to NO)
c
      rNO2_NO  = dt*(
     &       +          rk(  1)
     &       +          rk(  4)*yh(lO)
     &       + ( 0.200)*rk( 96)*yh(lISOP) )
c
c --- NO production terms except rNO2_NO
c     (totNO = initial NO + new production)
c
      totNO = y0(lNO) + dt*(
     &       + ( 0.110)*r( 14)
     &       +          r( 16)
     &       +          r( 23)
     &       +          r( 25) )
c
c --- Net loss of NO
c     (net either NO or NO2 produced)
c
      lsNO = 1.0 + dt*(
     &       +          rk( 21)*yh(lNO2)*H2O
     &       +          rk( 22)*yh(lOH)
     &       + ( 0.100)*rk( 64)*yh(lTO2)
     &       +          rk( 81)*yh(lXO2N) )
c
c --- Conversion of NO to NO2 except O3+NO (rO3wNO)
c
      rNO_NO2 = dt*(
     &       +          rk(  6)*yh(lO)
     &       +          rk( 15)*yh(lNO3)
     &       + ( 2.000)*rk( 20)*yh(lNO)
     &       +          rk( 28)*yh(lHO2)
     &       +          rk( 46)*yh(lC2O3)
     &       + ( 0.900)*rk( 64)*yh(lTO2)
     &       +          rk( 79)*yh(lXO2) )
c
c --- Remaining NO2 production
c
      totNO2 = y0(lNO2) + dt*(
     &       + ( 0.890)*r( 14)
     &       +          r( 15)
     &       +          r( 19)
     &       +          r( 24)
     &       +          r( 25)
     &       +          r( 30)
     &       +          r( 31)
     &       +          r( 48)
     &       +          r( 59)
     &       + ( 0.200)*r( 78) )
c
c --- Net loss of NO2
c     (net either NO or NO2 produced)
c
      lsNO2 = 1.0 + dt*(
     &       +          rk(  5)*yh(lO)
     &       +          rk(  7)*yh(lO3)
     &       +          rk( 17)*yh(lNO3)
     &       +          rk( 21)*yh(lNO)*H2O
     &       +          rk( 26)*yh(lOH)
     &       +          rk( 29)*yh(lHO2)
     &       +          rk( 47)*yh(lC2O3)
     &       +          rk( 55)*yh(lROR)
     &       +          rk( 68)*yh(lCRO)
     &       + ( 0.800)*rk( 96)*yh(lISOP) )
c
c --- Production of O3P except NO2+hv
c
      totO3P = y0(lO) + dt*(
     &       +          r(  8)
     &       +          r(  9)*yldO3P
     &       + ( 0.890)*r( 14) )
c
c --- Net loss of O3P
c
      lsO3P = 1.0 + dt*(
     &       +          rk(  2)
     &       +          rk(  4)*yh(lNO2)
     &       +          rk(  5)*yh(lNO2)
     &       +          rk(  6)*yh(lNO)
     &       +          rk( 40)*yh(lFORM)
     &       +          rk( 42)*yh(lALD2)
     &       +          rk( 56)*yh(lOLE)
     &       +          rk( 60)*yh(lETH)
     &       +          rk( 75)*yh(lISOP) )
c
c --- Production of O3 except O3P+O2
c
      totO3 = y0(lO3)
c
c --- Net loss of O3 except O3+NO (rO3wNO)
c
      lsO3 = 1.0 + dt*(
     &       +          rk(  7)*yh(lNO2)
     &       +          rk(  8)
     &       +          rk(  9)
     &       +          rk( 12)*yh(lOH)
     &       +          rk( 13)*yh(lHO2)
     &       +          rk( 58)*yh(lOLE)
     &       +          rk( 62)*yh(lETH)
     &       +          rk( 71)*yh(lOPEN)
     &       +          rk( 77)*yh(lISOP)
     &       +          rk( 93)*yh(lISPD) )
c
c --- Specific reactions
c
      jO3_O1D  = rk(  9)
      rNO2_O3P = dt*rk(  1)
      rO3P_O3  = dt*rk(  2)
      rO3wNO   = dt*rk(  3)
c
c --- Collect common terms
c
      t1 = rNO2_O3P / lsNO2
      t2 = rNO2_NO / lsNO2
      t3 = rNO_NO2 / lsNO
      t4 = rO3P_O3  / lsO3P
      t5 = t3*totNO - t2*totNO2
      f1 = 1.0 + t2 + t3
      f2 = t1*t4
      f3 = lsO3*lsNO + rO3wNO*totNO
      f4 = totO3 + totO3P * t4
c
c --- Solve for change in NO and NO2 
c
      A = rO3wNO * (f1 - f2)
      B = f1*f3 + rO3wNO*(f2*(totNO2 - totNO) + f4 + t5)
      C = rO3wNO*totNO*(f4 + totNO2 * f2) + f3*t5
      Q = -0.5 * (B + SIGN(1.0,B)*SQRT(B*B - 4.0*A*C))
      Q = MAX(Q/A ,C/Q)
c
c --- Update Concentrations
c
      y1(lNO)  = MAX(1.0E-25, (totNO + Q) / lsNO )
      y1(lNO2) = MAX(1.0E-25, (totNO2 - Q) / lsNO2 )
      s1 = totO3P + rNO2_O3P*y1(lNO2)
      s2 = t4*s1
      y1(lO3)  = MAX(1.0E-25, (totO3 + s2) / 
     &                             ( lsO3 + rO3wNO * y1(lNO) ) )
      y1(lO)   = MAX(1.0E-35, s1 / lsO3P )
      totO1D = jO3_O1D*y1(lO3)
      y1(lO1D) = MAX(1.0E-35, totO1D / lsO1D )
c
      return
      end

      subroutine hr_nxy3(y0,y1,yh,H2O,M,O2,CH4,H2,ny,rk,r,nr,dt)
c
c----CAMx v4.53 090209
c
c     HR_NXY3 solves NO3 and N2O5 using Hertel's equations
c
c     Copyright 1996-2009
c     ENVIRON International Corporation
c     Created by the CMC version 5
c
c --- Subroutines Called:
c        none
c
c --- Called by:
c        EBISOLV
c
c --- Argument definitions:
c        y0  - initial Conc for this step (ppm)
c        yh  - current Conc iteration (ppm)
c        y1  - next Conc iteration (ppm)
c        H2O - water vapor Conc (ppm)
c        M   - total gas Conc (ppm)
c        O2  - oxygen Conc (ppm)
c        CH4 - methane Conc (ppm)
c        H2  - hydrogen Conc (ppm)
c        ny  - dimension of y0, y1 and yh
c        rk  - rate constants (ppm-n hr-1)
c        r   - reaction rates (hr-1)
c        nr  - dimension of rk and r
c        dt  - time step (hr)
c
c --- Includes:
      implicit none
      include "camx.prm"
      include "ddmchm.com"
c
c --- Arguments:
      integer ny, nr
      real y0(ny+1), y1(ny+1), yh(ny+1)
      real rk(nr), r(nr)
      real H2O, M, O2, CH4, H2, dt
c
c --- Local variables:
      real N2
      real self, fwd, bck
      real newNO3, lsNO3, lsN2O5, totNO3, A, B, C, Q, t1
c
c --- Entry Point
c
      N2 = M - O2
c
      newNO3 = 
     &       +          r(  5)
     &       +          r(  7)
     &       +          r( 27) 
c
      lsN2O5 = 
     &       +          rk( 18)*H2O
     &       +          rk( 19) 
c
      lsNO3 = 
     &       +          rk( 14)
     &       +          rk( 15)*yh(lNO)
     &       +          rk( 16)*yh(lNO2)
     &       +          rk( 17)*yh(lNO2)
     &       +          rk( 41)*yh(lFORM)
     &       +          rk( 44)*yh(lALD2)
     &       +          rk( 59)*yh(lOLE)
     &       +          rk( 67)*yh(lCRES)
     &       +          rk( 78)*yh(lISOP)
     &       +          rk( 94)*yh(lISPD) 
c
      self = 0.0
      fwd = dt*( rk( 17) ) * yh(lNO2)
      bck = dt*( rk( 19) )
c
c --- Collect terms
c
      lsNO3   = 1.0 + dt*lsNO3
      lsN2O5  = 1.0 + dt*lsN2O5
      totNO3  = y0(lNO3) + dt*newNO3
      A = lsN2O5*totNO3 + bck*y0(lN2O5)
      B = lsNO3*y0(lN2O5) + fwd*totNO3
      C = (lsN2O5*lsNO3) - (fwd*bck)
c
c --- Update Concentrations
c
      y1(lNO3)  = MAX(1.0E-25, A/C )
      y1(lN2O5) = MAX(1.0E-25, B/C )
c
      return
      end

      subroutine hr_pan3(y0,y1,yh,H2O,M,O2,CH4,H2,ny,rk,r,nr,dt)
c
c----CAMx v4.53 090209
c
c     HR_PAN3 solves PAN and RCO3 using Hertel's equations
c
c     Copyright 1996-2009
c     ENVIRON International Corporation
c     Created by the CMC version 5
c
c --- Subroutines Called:
c        none
c
c --- Called by:
c        EBISOLV
c
c --- Argument definitions:
c        y0  - initial Conc for this step (ppm)
c        yh  - current Conc iteration (ppm)
c        y1  - next Conc iteration (ppm)
c        H2O - water vapor Conc (ppm)
c        M   - total gas Conc (ppm)
c        O2  - oxygen Conc (ppm)
c        CH4 - methane Conc (ppm)
c        H2  - hydrogen Conc (ppm)
c        ny  - dimension of y0, y1 and yh
c        rk  - rate constants (ppm-n hr-1)
c        r   - reaction rates (hr-1)
c        nr  - dimension of rk and r
c        dt  - time step (hr)
c
c --- Includes:
      implicit none
      include "camx.prm"
      include "ddmchm.com"
c
c --- Arguments:
      integer ny, nr
      real y0(ny+1), y1(ny+1), yh(ny+1)
      real rk(nr), r(nr)
      real H2O, M, O2, CH4, H2, dt
c
c --- Local variables:
      real N2
      real fwd, bck, self, A, B, C, Q
      real newRCO3, lsPAN, lsRCO3
c
c --- Entry Point
c
      N2 = M - O2
c
      newRCO3 = 
     &       +          r( 42)
     &       +          r( 43)
     &       +          r( 44)
     &       +          r( 69)
     &       +          r( 70)
     &       + ( 0.620)*r( 71)
     &       +          r( 73)
     &       +          r( 74)
     &       + ( 0.250)*r( 75)
     &       + ( 0.200)*r( 77)
     &       + ( 0.498)*r( 92)
     &       + ( 0.114)*r( 93)
     &       + ( 0.075)*r( 94)
     &       + ( 0.967)*r( 95) 
c
      lsRCO3 = 
     &       +          rk( 46)*yh(lNO)
     &       +          rk( 47)*yh(lNO2)
     &       +          rk( 50)*yh(lHO2) 
c
      lsPAN = 
     &       + rk( 48)
c
      fwd  = dt*( rk( 47) ) * yh(lNO2)
      bck  = dt*( rk( 48) )
      self = dt*rk( 49)
      lsPAN  = 1.0 + dt*lsPAN
      lsRCO3 = 1.0 + dt*lsRCO3
c
c-----Solve for peroxyacetyl radical
c
      A = 2.0 * self * lsPAN
      B = ( lsPAN*lsRCO3) - (fwd*bck)
      C = lsPAN*( y0(lC2O3) + newRCO3*dt ) + bck*y0(lPAN)
      Q = -0.5 * ( B + SIGN(1.0,B)*SQRT(B*B + 4.0*A*C) )
c
c --- Update Concentrations
c
      y1(lC2O3) = MAX(1.0E-25, MAX(Q/A, -C/Q) )
      y1(lPAN)  = MAX(1.0E-25, (y0(lPAN) + fwd*y1(lC2O3)) / lsPAN )
c
      return
      end

